unit Display;

interface

uses
  Windows, Messages, SysUtils, Classes, GR32, GR32_Image, GR32_Layers,
  Controls, Forms, Dialogs, Menus, ExtCtrls, ComCtrls;

const
  InteractingStatesBaseRule = 'TB0,TS1,B1;00002030,B2;00000000,B3;02030000,' +
    'B4;00020000,B5;00200010,B6;00101000,B7;00020003,B8;02000300,S1;00000000,' +
    'S2;00010020,S3;10000000,S4;20000000,S5;00200000,S6;20100000,S7;00022000,' +
    'S8;00200001';
  WeightedGenerationsBaseRule = 'C4,SW0;1;0;0,PW1;1;1;1;0;1;1;1;1,RS3;4;5,RB2';

type
  TDisplayForm = class(TForm)
    Panel: TPanel;
    StatusBar: TStatusBar;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    OpenMenu: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveMenu: TMenuItem;
    SaveDialog: TSaveDialog;
    OptionsMenu: TMenuItem;
    RulesMenu: TMenuItem;
    WeightedGenerationsMenu: TMenuItem;
    InteractingStatesMenu: TMenuItem;
    LoadRuleMenu: TMenuItem;
    NewRuleMenu: TMenuItem;
    AnalyzeMenu: TMenuItem;
    ClearMenu: TMenuItem;
    RandomMenu: TMenuItem;
    StartMenu: TMenuItem;
    StepMenu: TMenuItem;
    AboutMenu: TMenuItem;
    Image: TImage32;
    GeneralBinaryMenu: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OpenMenuClick(Sender: TObject);
    procedure SaveMenuClick(Sender: TObject);
    procedure OptionsMenuClick(Sender: TObject);
    procedure GeneralBinaryMenuClick(Sender: TObject);
    procedure InteractingStatesMenuClick(Sender: TObject);
    procedure WeightedGenerationsMenuClick(Sender: TObject);
    procedure LoadRuleMenuClick(Sender: TObject);
    procedure NewRuleMenuClick(Sender: TObject);
    procedure AnalyzeMenuClick(Sender: TObject);
    procedure ClearMenuClick(Sender: TObject);
    procedure RandomMenuClick(Sender: TObject);
    procedure StartMenuClick(Sender: TObject);
    procedure StepMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);
    procedure ImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer; Layer: TCustomLayer);
  private
    procedure ResetFrameCount;
    procedure RemoveSelection;
    procedure UpdateStateSelector;
  public
  end;

  TRule = (GeneralBinaryRule, InteractingStatesRule, WeightedGenerationsRule);

var
  DisplayForm: TDisplayForm;
  StopNow: Boolean;
  MaxState: Integer;
  Rule: TRule;
  GeneralBinaryBaseRule, GeneralBinaryRuleString: String;
  WeightedGenerationsRuleString, InteractingStatesRuleString: String;
  FrameCount: Integer;

implementation

uses
  Options, GeneralBinary, InteractingStates, WeightedGenerations, LoadRule,
  StateSelector, SelectionControl, About, SquareCA;

{$R *.DFM}

procedure TDisplayForm.FormCreate(Sender: TObject);
begin
  DisplayWidth := Image.Width;
  DisplayHeight := Image.Height;
  InitializeCA;
  CreateCells;
  ResetFrameCount;
  GeneralBinaryBaseRule := 'C2,NM,S5abababababbaa4babaabbab4ababbabbabbababaa' +
    '5baabab6a4bababababbaabaabaababb7abba3bab7abab16a4baa4baba3baabaabbaab4a' +
    '4b3abab5ab7ab8abbab3abbaab3ab3ab11abab5ab23a,B7ab3abaabb7abaa6b3abaabba7' +
    'baabbaaba5babb7abaabbaab6ababaababbabaabbaaba5ba5babbabba5b4abaabba7baab' +
    'baaba5babba10babba6ba4ba4baa3ba8babbabbaababbabbabba5ba4ba4ba4babba3baba' +
    'ab';
  GeneralBinaryRuleString := GeneralBinaryBaseRule;
  GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
  WeightedGenerationsRuleString := WeightedGenerationsBaseRule;
  WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
  InteractingStatesRuleString := InteractingStatesBaseRule;
  InteractingStates.InitRuleFromString(InteractingStatesRuleString);
  MaxState := 3;
  SelectedState := 1;
  Rule := WeightedGenerationsRule;
end;

procedure TDisplayForm.FormResize(Sender: TObject);
begin
  if ClientWidth < 250 then
    ClientWidth := 250;
  if ClientHeight < 250 then
    ClientHeight := 250;
  DisplayWidth := Image.Width;
  DisplayHeight := Image.Height;
  Image.Bitmap.Width := DisplayWidth;
  Image.Bitmap.Height := DisplayHeight;
  CreateBitmap;
  SetSize;
  DestroyCells;
  CreateCells;
  ResetFrameCount;
  DrawCells;
end;

procedure TDisplayForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  StartMenu.caption := '&Start';
  StopNow := True;
end;

procedure TDisplayForm.RemoveSelection;
begin
  if Selected <> None then
  begin
    Selected := Removing;
    DrawCells;
    ResetNewStates;
  end;
end;

procedure TDisplayForm.UpdateStateSelector;
begin
    with StateSelectorDialog do
    if Visible then
    begin
      Close;
      Show;
    end;
end;

procedure TDisplayForm.OpenMenuClick(Sender: TObject);
begin
  RemoveSelection;
  OpenDialog.InitialDir := 'C:\New Rules';
  if OpenDialog.Execute then
  begin
    LoadPattern(OpenDialog.FileName);
    if Rule = GeneralBinaryRule then
    begin
      GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
      MaxState := BinaryRuleStates - 1;
    end
    else if Rule = InteractingStatesRule then
    begin
      InteractingStates.InitRuleFromString(InteractingStatesRuleString);
      MaxState := 8;
    end
    else if Rule = WeightedGenerationsRule then
    begin
      WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
      MaxState := NumberOfStates - 1;
    end;
    SelectedState := 1;
    UpdateStateSelector;
    DrawCells;
  end;
end;

procedure TDisplayForm.SaveMenuClick(Sender: TObject);
begin
  RemoveSelection;
  SaveDialog.InitialDir := 'C:\New Rules';
  if SaveDialog.Execute then
  begin
    SelectAll;
    Selected := None;
    SavePattern(SaveDialog.FileName);
  end;
end;

procedure TDisplayForm.ResetFrameCount;
begin
  FrameCount := 0;
  Statusbar.Panels[0].Text := IntToStr(FrameCount);
end;

procedure TDisplayForm.OptionsMenuClick(Sender: TObject);
var
  OldIndex, NewIndex: Integer;
begin
  RemoveSelection;
  OldIndex := OptionsDialog.GridSizeSelector.ItemIndex;
  if OptionsDialog.ShowModal = mrOk then
  begin
    NewIndex := OptionsDialog.GridSizeSelector.ItemIndex;
    if NewIndex <> OldIndex then
    begin
      GridSize := StrToInt(OptionsDialog.GridSizeSelector.Items[NewIndex]);
      CreateBitmap;
      SetSize;
      DestroyCells;
      CreateCells;
      ResetFrameCount;
    end
    else
      InvalidateStates;
    UpdateStateSelector;
    DrawCells;
  end;
end;

procedure TDisplayForm.GeneralBinaryMenuClick(Sender: TObject);
begin
  RemoveSelection;
  GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
  if GeneralBinaryDialog.ShowModal = mrOK then
  begin
    GeneralBinaryRuleString := GeneralBinary.GetRuleAsString;
    Rule := GeneralBinaryRule;
    MaxState := BinaryRuleStates - 1;
    SelectedState := 1;
    ClearMenuClick(Self);
    UpdateStateSelector;
  end
  else
    GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
end;

procedure TDisplayForm.InteractingStatesMenuClick(Sender: TObject);
begin
  RemoveSelection;
  InteractingStates.InitRuleFromString(InteractingStatesRuleString);
  if InteractingStatesDialog.ShowModal = mrOk then
  begin
    InteractingStatesRuleString := InteractingStates.GetRuleAsString;
    Rule := InteractingStatesRule;
    MaxState := 8;
    SelectedState := 1;
    ClearMenuClick(Self);
    UpdateStateSelector;
  end
  else
    InteractingStates.InitRuleFromString(InteractingStatesRuleString);
end;

procedure TDisplayForm.WeightedGenerationsMenuClick(Sender: TObject);
begin
  RemoveSelection;
  WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
  if WeightedGenerationsDialog.ShowModal = mrOk then
  begin
    WeightedGenerationsRuleString := WeightedGenerations.GetRuleAsString;
    Rule := WeightedGenerationsRule;
    MaxState := NumberOfStates - 1;
    SelectedState := 1;
    ClearMenuClick(Self);
    UpdateStateSelector;
  end
  else
    WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
end;

procedure TDisplayForm.LoadRuleMenuClick(Sender: TObject);
begin
  RemoveSelection;
  if LoadRuleDialog.ShowModal = mrOk then
  begin
    with LoadRuleDialog do
    begin
      if GeneralBinaryRadioButton.Checked then
      begin
        Rule := GeneralBinaryRule;
        GeneralBinaryRuleString := RuleStringEdit.Text;
        if GeneralBinaryRuleString[2] = '0' then
          GeneralBinaryRuleString[2] := '2';
        GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
        MaxState := BinaryRuleStates - 1;
      end
      else if InteractingStatesRadioButton.Checked then
      begin
        Rule := InteractingStatesRule;
        InteractingStatesRuleString := RuleStringEdit.Text;
        InteractingStates.InitRuleFromString(InteractingStatesRuleString);
        MaxState := 8;
      end
      else if WeightedGenerationsRadioButton.Checked then
      begin
        Rule := WeightedGenerationsRule;
        WeightedGenerationsRuleString := RuleStringEdit.Text;
        WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
        MaxState := NumberOfStates - 1;
      end;
    end;
    SelectedState := 1;
    ClearMenuClick(Self);
    UpdateStateSelector;
  end;
end;

procedure TDisplayForm.NewRuleMenuClick(Sender: TObject);
begin
  RemoveSelection;
  case Rule of
    GeneralBinaryRule:
    begin
      GeneralBinary.InitRuleFromString(GeneralBinaryRuleString);
      GeneralBinaryDialog.GenerateNewRule;
      MaxState := BinaryRuleStates - 1;
      GeneralBinaryRuleString := GeneralBinary.GetRuleAsString;
    end;
    InteractingStatesRule:
    begin
      InteractingStates.InitRuleFromString(InteractingStatesRuleString);
      InteractingStatesDialog.GenerateNewRule;
      MaxState := 8;
      InteractingStatesRuleString := InteractingStates.GetRuleAsString;
    end;
    WeightedGenerationsRule:
    begin
      WeightedGenerations.InitRuleFromString(WeightedGenerationsRuleString);
      WeightedGenerationsDialog.GenerateNewRule;
      MaxState := NumberOfStates - 1;
      WeightedGenerationsRuleString := WeightedGenerations.GetRuleAsString;
    end;
    else
      Exit;
  end;
  SelectedState := 1;
  RandomMenuClick(Self);
  UpdateStateSelector;
end;

procedure TDisplayForm.AnalyzeMenuClick(Sender: TObject);
begin
  RemoveSelection;
  if StartMenu.Caption = '&Stop' then
    Exit;
  if AnalyzeMenu.Caption = '&Analyze' then
  begin
    AnalyzeMenu.Caption := 'S&top';
    StopNow := False;
    SaveState;
    repeat
      Application.ProcessMessages;
      Step;
      if CompareState then
      begin
        AnalyzeMenu.Caption := '&Analyze';
        StopNow := True;
      end;
    until StopNow = True;
    InvalidateStates;
    DrawCells;
  end
  else
  begin
    AnalyzeMenu.Caption := '&Analyze';
    StopNow := True;
  end;
end;

procedure TDisplayForm.ClearMenuClick(Sender: TObject);
begin
  RemoveSelection;
  ClearCells;
  ResetFrameCount;
  DrawCells;
end;

procedure TDisplayForm.RandomMenuClick(Sender: TObject);
begin
  RemoveSelection;
  RandomCells;
  ResetFrameCount;
  DrawCells;
end;

procedure TDisplayForm.StartMenuClick(Sender: TObject);
begin
  RemoveSelection;
  if AnalyzeMenu.Caption = 'S&top' then
    Exit;
  if StartMenu.Caption = '&Start' then
  begin
    StartMenu.Caption := '&Stop';
    StopNow := False;
    repeat
      Application.ProcessMessages;
      Step;
      DrawCells;
    until StopNow = True;
  end
  else
  begin
    StartMenu.Caption := '&Start';
    StopNow := True;
  end;
end;

procedure TDisplayForm.StepMenuClick(Sender: TObject);
var
  I, StepSize: Integer;
begin
  RemoveSelection;
  StepSize := StrToInt(OptionsDialog.StepSizeEdit.Text);
  for I := 1 to StepSize do
  begin
    Step;
    DrawCells;
  end;
end;

procedure TDisplayForm.AboutMenuClick(Sender: TObject);
begin
  RemoveSelection;
  AboutDialog.ShowModal;
end;

procedure TDisplayForm.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
begin
  if Button = mbRight then
    if ssShift in Shift then
      StateSelectorDialog.Show
    else
      SelectionControlDialog.Show
  else
  begin
    if SelectionMoving then
    begin
      StopMovingSelectedCells(Shift, X, Y);
      DrawCells;
    end
    else
    begin
      ProcessMouseClick(X, Y, Shift);
      DrawCells;
    end;
  end;
end;

procedure TDisplayForm.ImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
  Layer: TCustomLayer);
begin
  if (ssCtrl in Shift) or (Button = mbRight) then
    Exit;
  if (Selected = GroupSelected) then
  begin
    StartMovingSelectedCells(Shift, X, Y);
    if SelectionControlDialog.MoveRadioButton.Checked then
      ClearInside;
  end;
end;

procedure TDisplayForm.ImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer; Layer: TCustomLayer);
begin
  if SelectionMoving then
  begin
    MoveSelectedCells(X, Y);
  end;
end;


end.

