unit GeneralBinary;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, Math, ComCtrls;

const
  NeighborConversion: array[0..2, 0..2] of Integer =
    ((0, 1, 2), (7, 8, 3), (6, 5, 4));
  DensityConversion: array[1..10, 1..2] of Integer =
    ((65, 3), (50, 4), (40, 5), (30, 7), (25, 8), (20, 10), (15, 13), (10, 20),
     (5, 35), (2, 83));

type
  TGeneralBinaryDialog = class(TForm)
    NumberStatesGroup: TGroupBox;
    NumberStatesEdit: TEdit;
    NumberStatesUpDown: TUpDown;
    ControlGroup: TGroupBox;
    NoneRadioButton: TRadioButton;
    BirthRadioButton: TRadioButton;
    SurviveRadioButton: TRadioButton;
    BothRadioButton: TRadioButton;
    NeighborGroup: TGroupBox;
    NeighborGrid: TDrawGrid;
    RuleGroup: TGroupBox;
    RuleGrid: TDrawGrid;
    ClearButton: TButton;
    IsotropicButton: TButton;
    RandomButton: TButton;
    RuleText: TEdit;
    OkButton: TButton;
    CancelButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure NeighborGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure NeighborGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure RuleGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure RuleGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure NumberStatesEditChange(Sender: TObject);
    procedure NoneRadioButtonClick(Sender: TObject);
    procedure BirthRadioButtonClick(Sender: TObject);
    procedure SurviveRadioButtonClick(Sender: TObject);
    procedure BothRadioButtonClick(Sender: TObject);
    procedure SetNumberStates;
    procedure SetControlButtons;
    procedure GenerateNewRule;
    procedure ClearButtonClick(Sender: TObject);
    procedure IsotropicButtonClick(Sender: TObject);
    procedure RandomButtonClick(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
  public
  end;

var
  GeneralBinaryDialog: TGeneralBinaryDialog;
  GeneralBinaryRules: array[0..15, 0..15] of Integer;
  BinaryRuleStates: Integer;
  Neighbors: array[0..7] of Integer;
  RowSelected, ColumnSelected: Integer;

procedure InitRuleFromString(S: String);
function GetRuleAsString: String;

implementation

uses
  Options, Display;

{$R *.DFM}

procedure TGeneralBinaryDialog.FormShow(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to 7 do
    Neighbors[I] := 0;
  RowSelected := 0;
  ColumnSelected := 0;
  SetNumberStates;
  SetControlButtons;
  RuleText.Text := GetRuleAsString;
end;

procedure TGeneralBinaryDialog.NeighborGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  if (ACol = 1) and (ARow = 1) then
    Exit;
  with (Sender as TDrawGrid) do
  begin
    if Neighbors[NeighborConversion[ARow, ACol]] = 0 then
      Canvas.Pen.Color := clWhite
    else
      Canvas.Pen.Color := $CBC0FF;
    Canvas.Brush.Color := Canvas.Pen.Color;
    Canvas.Rectangle(Rect);
  end;
end;

procedure TGeneralBinaryDialog.NeighborGridSelectCell(Sender: TObject;
  ACol, ARow: Integer; var CanSelect: Boolean);
var
  I, N, P: Integer;
begin
  I := NeighborConversion[ARow, ACol];
  Neighbors[I] := Neighbors[I] xor 1;
  if Neighbors[0] = 1 then
    N := 128
  else
    N := 0;
  P := 1;
  for I := 1 to 7 do
  begin
    Inc(N, Neighbors[I] * P);
    P := P * 2;
  end;
  RowSelected := N div 16;
  ColumnSelected := N mod 16;
  SetControlButtons;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.RuleGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TDrawGrid) do
  begin
    if GeneralBinaryRules[Acol, ARow] = 0 then
      Canvas.Brush.Color := clWhite
    else if GeneralBinaryRules[ACol, ARow] = 1 then
      Canvas.Brush.Color := clAqua
    else if GeneralBinaryRules[ACol, ARow] = 2 then
      Canvas.Brush.Color := clYellow
    else if GeneralBinaryRules[ACol, ARow] = 3 then
      Canvas.Brush.Color := $2FFFAD;
    if (ARow = RowSelected) and (ACol = ColumnSelected) then
      Canvas.Pen.Color := clBlack
    else
      Canvas.Pen.Color := Canvas.Brush.Color;
    Canvas.Rectangle(Rect);
  end;
end;

procedure TGeneralBinaryDialog.RuleGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var
  I, N: Integer;
begin
  RowSelected := ARow;
  ColumnSelected := ACol;
  N := ARow * 16 + ACol;
  if (N and 128) <> 0 then
    Neighbors[0] := 1
  else
    Neighbors[0] := 0;
  for I := 7 downto 1 do
  begin
    N := N shl 1;
    if (N and 128) <> 0 then
      Neighbors[I] := 1
    else
      Neighbors[I] := 0;
  end;
  SetControlButtons;
  RuleGrid.Invalidate;
  NeighborGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.NumberStatesEditChange(Sender: TObject);
begin
  if GeneralBinaryDialog.Active then
  begin
    BinaryRuleStates := StrToInt(NumberStatesEdit.Text);
    RuleText.Text := GetRuleAsString;
  end;    
end;

procedure TGeneralBinaryDialog.NoneRadioButtonClick(Sender: TObject);
begin
  GeneralBinaryRules[ColumnSelected, RowSelected] := 0;
  RuleText.Text := GetRuleAsString;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.BirthRadioButtonClick(Sender: TObject);
begin
  GeneralBinaryRules[ColumnSelected, RowSelected] := 1;
  RuleText.Text := GetRuleAsString;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.SurviveRadioButtonClick(Sender: TObject);
begin
  GeneralBinaryRules[ColumnSelected, RowSelected] := 2;
  RuleText.Text := GetRuleAsString;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.BothRadioButtonClick(Sender: TObject);
begin
  GeneralBinaryRules[ColumnSelected, RowSelected] := 3;
  RuleText.Text := GetRuleAsString;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.SetNumberStates;
begin
  with GeneralBinaryDialog.NumberStatesEdit do
  begin
    ReadOnly := False;
    Text := IntToStr(BinaryRuleStates);
    ReadOnly := True;
  end;
end;

procedure TGeneralBinaryDialog.SetControlButtons;
begin
  if GeneralBinaryRules[ColumnSelected, RowSelected] = 0 then
    NoneRadioButton.Checked := True
  else if GeneralBinaryRules[ColumnSelected, RowSelected] = 1 then
    BirthRadioButton.Checked := True
  else if GeneralBinaryRules[ColumnSelected, RowSelected] = 2 then
    SurviveRadioButton.Checked := True
  else if GeneralBinaryRules[ColumnSelected, RowSelected] = 3 then
    BothRadioButton.Checked := True;
end;

procedure TGeneralBinaryDialog.GenerateNewRule;
var
  I, J: Integer;
  R: Integer;
  Density: Integer;
begin
  Density := StrToInt(OptionsDialog.GeneralBinaryDensitySelector.Items
                        [OptionsDialog.GeneralBinaryDensitySelector.ItemIndex]);
  for I := 1 to 10 do
    if Density = DensityConversion[I, 1] then
    begin
      Density := DensityConversion[I, 2];
      break;
    end;
  if OptionsDialog.GeneralBinaryVariant.Checked then
  begin
    with DisplayForm do
    begin
      GeneralBinaryRuleString := GeneralBinaryBaseRule;
      InitRuleFromString(GeneralBinaryRuleString);
    end;
    for I := 0 to 15 do
      for J := 0 to 15 do
      begin
        R := Random(Density);
        if R > 2 then
          R := 0;
        GeneralBinaryRules[J, I] := GeneralBinaryRules[J, I] or R;
      end;
  end
  else
  begin
    for I := 0 to 15 do
      for J := 0 to 15 do
      begin
        R := Random(Density);
        if R <> 1 then
          R := 0;
        GeneralBinaryRules[J, I] := R;
        R := Random(Density);
        if R <> 2 then
          R := 0;
        GeneralBinaryRules[J, I] := GeneralBinaryRules[J, I] or R;
      end;
  end;
  if OptionsDialog.GeneralBinaryIsotropic.Checked then
    IsotropicButtonClick(Self);
end;

procedure TGeneralBinaryDialog.ClearButtonClick(Sender: TObject);
var
  I, J: Integer;
begin
  for I := 0 to 15 do
    for J := 0 to 15 do
      GeneralBinaryRules[J, I] := 0;
  RuleText.Text := GetRuleAsString;
  SetControlButtons;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.IsotropicButtonClick(Sender: TObject);
var
  I, J, K, L, N, P: Integer;

procedure RotateNeighbors;
var
  I: Integer;
  T: array[0..7] of Integer;
begin
  for I := 0 to 7 do
    T[I] := Neighbors[(I + 2) mod 8];
  for I := 0 to 7 do
    Neighbors[I] := T[I];
end;

procedure ReflectNeighbors;

procedure Swap(I, J: Integer);
var
  T: Integer;
begin
  T := Neighbors[I];
  Neighbors[I] := Neighbors[J];
  Neighbors[J] := T;
end;

begin
  Swap(0, 2);
  Swap(7, 3);
  Swap(6, 4);
end;

begin
  for I := 0 to 15 do
    for J := 0 to 15 do
    begin
      N := J * 16 + I;
      if (N and 128) <> 0 then
        Neighbors[0] := 1
      else
        Neighbors[0] := 0;
      for K := 7 downto 1 do
      begin
        N := N shl 1;
        if (N and 128) <> 0 then
          Neighbors[K] := 1
        else
          Neighbors[K] := 0;
      end;
      for K := 1 to 7 do
      begin
        if K = 4 then
          ReflectNeighbors
        else
          RotateNeighbors;
        if Neighbors[0] = 1 then
          N := 128
        else
          N := 0;
        P := 1;
        for L := 1 to 7 do
        begin
          Inc(N, Neighbors[L] * P);
          P := P * 2;
        end;
        GeneralBinaryRules[N div 16, N mod 16] :=
          GeneralBinaryRules[N div 16, N mod 16] or GeneralBinaryRules[J, I];
      end;
    end;
  RowSelected := 15;
  ColumnSelected := 15;
  RuleText.Text := GetRuleAsString;
  SetControlButtons;
  RuleGrid.Invalidate;
  NeighborGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.RandomButtonClick(Sender: TObject);
begin
  GenerateNewRule;
  RuleText.Text := GetRuleAsString;
  SetControlButtons;
  RuleGrid.Invalidate;
end;

procedure TGeneralBinaryDialog.OkButtonClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TGeneralBinaryDialog.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure InitRuleFromString(S: String);
var
  Chr: Char;
  Count: Integer;
  Index: Integer;
  I, J: Integer;

procedure GetCount;
begin
  Count := Ord(S[Index]) - 48;
  Inc(Index);
  while (S[Index] >= '0') and (S[Index] <= '9') do
  begin
    Count := Count * 10 + Ord(S[Index]) - 48;
    Inc(Index);
  end;
end;

procedure GetNextItem;
begin
  if (S[Index] < '0') or (S[Index] > '9') then
    Count := 1
  else
    GetCount;
  Chr := S[Index];
  Inc(Index);
end;

begin
  Index := 2;
  GetNextItem;
  BinaryRuleStates := Count;
  while Chr <> 'S' do
    GetNextItem;
  Count := 0;
  for I := 0 to 15 do
    for J := 0 to 15 do
    begin
      if Count = 0 then
        GetNextItem;
      GeneralBinaryRules[J, I] := (Ord(Chr) - 97) shl 1;
      Dec(Count);
    end;
  while Chr <> 'B' do
    GetNextItem;
  Count := 0;
  for I := 0 to 15 do
    for J := 0 to 15 do
    begin
      if Count = 0 then
        GetNextItem;
      GeneralBinaryRules[J, I] := GeneralBinaryRules[J, I] or (Ord(Chr) - 97);
      Dec(Count);
    end;
end;

function GetRuleAsString: String;
var
  S: String;
  I, J: Integer;
  Count, State: Integer;

procedure AddItem;
begin
  if Count = 0 then
    Exit;
  if Count > 2 then
    S := S + IntToStr(Count) + Chr(State + 97)
  else
  begin
    S := S + Chr(State + 97);
    if Count = 2 then
      S := S + Chr(State + 97);
  end;
end;

begin
  S := 'C' + IntToStr(BinaryRuleStates) + ',NM,S';
  Count := 0;
  State := 0;
  for I := 0 to 15 do
    for J := 0 to 15 do
    begin
      if ((GeneralBinaryRules[J, I] and 2) shr 1) = State then
        Inc(Count)
      else
      begin
        AddItem;
        Count := 1;
        State := GeneralBinaryRules[J, I] shr 1;
      end;
    end;
  AddItem;
  S := S + ',B';
  Count := 0;
  State := 0;
  for I := 0 to 15 do
    for J := 0 to 15 do
    begin
      if (GeneralBinaryRules[J, I] and 1) = State then
        Inc(Count)
      else
      begin
        AddItem;
        Count := 1;
        State := GeneralBinaryRules[J, I] and 1;
      end;
    end;
  AddItem;
  Result := S;
end;

end.

