unit InteractingStates;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Grids;

const
  ArraySize = 8;

type
  RuleSpecification = array[1..ArraySize, 1..ArraySize] of Integer;
  TestOperator = (OR_GE, OR_EQ, AND_GE, AND_EQ);

  TInteractingStatesDialog = class(TForm)
    BirthControl: TGroupBox;
    BirthGrid: TStringGrid;
    BirthColumnLabel: TLabel;
    BirthRowLabel1: TLabel;
    BirthRowLabel2: TLabel;
    BirthRowLabel3: TLabel;
    BirthRowLabel4: TLabel;
    BirthRowLabel5: TLabel;
    BirthRowLabel6: TLabel;
    BirthRowLabel7: TLabel;
    BirthRowLabel8: TLabel;
    BirthOperatorSelector: TButton;
    BirthResetButton: TButton;
    BirthRandomButton: TButton;
    BirthRandomLimitEdit: TLabel;
    SurvivalControl: TGroupBox;
    SurvivalGrid: TStringGrid;
    SurvivalColumnLabel: TLabel;
    SurvivalRowLabel1: TLabel;
    SurvivalRowLabel2: TLabel;
    SurvivalRowLabel3: TLabel;
    SurvivalRowLabel4: TLabel;
    SurvivalRowLabel5: TLabel;
    SurvivalRowLabel6: TLabel;
    SurvivalRowLabel7: TLabel;
    SurvivalRowLabel8: TLabel;
    SurvivalOperatorSelector: TButton;
    SurvivalResetButton: TButton;
    SurvivalRandomButton: TButton;
    SurvivalRandomLimitEdit: TLabel;
    RuleDisplay: TEdit;
    OKButton: TButton;
    CancelButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure InitOperatorSelectors;
    procedure InitRuleGrids;
    procedure BirthGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure SurvivalGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure BirthGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure SurvivalGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BirthOperatorSelectorClick(Sender: TObject);
    procedure BirthResetButtonClick(Sender: TObject);
    procedure BirthRandomLimitEditClick(Sender: TObject);
    procedure BirthRandomButtonClick(Sender: TObject);
    procedure SurvivalOperatorSelectorClick(Sender: TObject);
    procedure SurvivalResetButtonClick(Sender: TObject);
    procedure SurvivalRandomLimitEditClick(Sender: TObject);
    procedure SurvivalRandomButtonClick(Sender: TObject);
    procedure GenerateNewRule;
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
  public
  end;

var
  InteractingStatesDialog: TInteractingStatesDialog;
  BirthRandomLimit: Integer;
  SurvivalRandomLimit: Integer;
  Birth: RuleSpecification;
  Survival: RuleSpecification;
  BirthTestOperator: TestOperator;
  SurvivalTestOperator: TestOperator;
  DecCells: Boolean;

procedure InitBirths;
procedure InitSurvivals;
procedure InitVars;
procedure InitRuleFromString(S: String);
function GetRuleAsString: String;

implementation

uses
  Options;

{$R *.DFM}

procedure TInteractingStatesDialog.FormCreate(Sender: TObject);
begin
  BirthRandomLimit := StrToInt(BirthRandomLimitEdit.Caption) + 1;
  SurvivalRandomLimit := StrToInt(SurvivalRandomLimitEdit.Caption) + 1
end;

procedure TInteractingStatesDialog.FormShow(Sender: TObject);
begin
  InitOperatorSelectors;
  InitRuleGrids
end;

procedure TInteractingStatesDialog.InitOperatorSelectors;
begin
  case BirthTestOperator of
    OR_GE: BirthOperatorSelector.Caption := 'OR     >=';
    OR_EQ: BirthOperatorSelector.Caption := 'OR      =';
    AND_GE: BirthOperatorSelector.Caption := 'AND   >=';
    AND_EQ: BirthOperatorSelector.Caption := 'AND    ='
  end;
  case SurvivalTestOperator of
    OR_GE: SurvivalOperatorSelector.Caption := 'OR     >=';
    OR_EQ: SurvivalOperatorSelector.Caption := 'OR      =';
    AND_GE: SurvivalOperatorSelector.Caption := 'AND   >=';
    AND_EQ: SurvivalOperatorSelector.Caption := 'AND    ='
  end
end;

procedure TInteractingStatesDialog.InitRuleGrids;
var
  I, J: Integer;
begin
  with BirthGrid do
    for I := 0 to ColCount - 1 do
      for J := 0 to RowCount - 1 do
        Cells[I, J] := IntToStr(Birth[J + 1, I + 1]);
  with SurvivalGrid do
    for I := 0 to ColCount - 1 do
      for J := 0 to RowCount - 1 do
        SurvivalGrid.Cells[I, J] := IntToStr(Survival[J + 1, I + 1]);
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInteractingStatesDialog.BirthGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  with BirthGrid do
  begin
    if DecCells = true then
      if Cells[ACol, ARow] = '0' then
        Cells[ACol, ARow] := '8'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) - 1)
    else
      if Cells[ACol, ARow] = '8' then
        Cells[ACol, ARow] := '0'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) + 1);
    Birth[ARow + 1, ACol + 1] := StrToInt(Cells[ACol, ARow])
  end;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInteractingStatesDialog.SurvivalGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  with SurvivalGrid do
  begin
    if DecCells = true then
      if Cells[ACol, ARow] = '0' then
        Cells[ACol, ARow] := '8'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) - 1)
    else
      if Cells[ACol, ARow] = '8' then
        Cells[ACol, ARow] := '0'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) + 1);
    Survival[ARow + 1, ACol + 1] := StrToInt(Cells[ACol, ARow])
  end;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInteractingStatesDialog.BirthGridDrawCell(Sender: TObject;
  ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TStringGrid) do
  begin
    Canvas.TextRect(Rect, Rect.Left + 6, Rect.Top + 3, cells[ACol, ARow])
  end
end;

procedure TInteractingStatesDialog.SurvivalGridDrawCell(Sender: TObject;
  ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TStringGrid) do
  begin
    Canvas.TextRect(Rect, Rect.Left + 6, Rect.Top + 3, cells[ACol, ARow])
  end
end;

procedure TInteractingStatesDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ssShift in Shift then
    DecCells := true
end;

procedure TInteractingStatesDialog.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  DecCells := false
end;

procedure TInteractingStatesDialog.BirthOperatorSelectorClick(Sender: TObject);
begin
  if BirthTestOperator = AND_EQ then
    BirthTestOperator := OR_GE
  else
    BirthTestOperator := Succ(BirthTestOperator);
  InitOperatorSelectors;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInteractingStatesDialog.BirthRandomLimitEditClick(Sender: TObject);
begin
  with BirthRandomLimitEdit do
  begin
    if DecCells = true then
      if Caption = '1' then
        Caption := '8'
      else
        Caption := IntToStr(StrToInt(Caption) - 1)
    else
      if Caption = '8' then
        Caption := '1'
      else
        Caption := IntToStr(StrToInt(Caption) + 1);
    BirthRandomLimit := StrToInt(Caption) + 1
  end
end;

procedure TInteractingStatesDialog.BirthResetButtonClick(Sender: TObject);
begin
  InitBirths;
  InitRuleGrids
end;

procedure TInteractingStatesDialog.BirthRandomButtonClick(Sender: TObject);
var
  I: Integer;
begin
  Randomize;
  for I := 1 to ArraySize do
  begin
    Birth[I, Random(ArraySize) + 1] := Random(BirthRandomLimit);
    Birth[I, Random(ArraySize) + 1] := Random(BirthRandomLimit)
  end;
  InitRuleGrids
end;

procedure TInteractingStatesDialog.SurvivalOperatorSelectorClick
                                                              (Sender: TObject);
begin
  if SurvivalTestOperator = AND_EQ then
    SurvivalTestOperator := OR_GE
  else
    SurvivalTestOperator := Succ(SurvivalTestOperator);
  InitOperatorSelectors;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInteractingStatesDialog.SurvivalRandomLimitEditClick
                                                              (Sender: TObject);
begin
  with SurvivalRandomLimitEdit do
  begin
    if DecCells = true then
      if Caption = '1' then
        Caption := '8'
      else
        Caption := IntToStr(StrToInt(Caption) - 1)
    else
      if Caption = '8' then
        Caption := '1'
      else
        Caption := IntToStr(StrToInt(Caption) + 1);
    SurvivalRandomLimit := StrToInt(Caption) + 1
  end
end;

procedure TInteractingStatesDialog.SurvivalResetButtonClick(Sender: TObject);
begin
  InitSurvivals;
  InitRuleGrids
end;

procedure TInteractingStatesDialog.SurvivalRandomButtonClick(Sender: TObject);
var
  I: Integer;
begin
  Randomize;
  for I := 1 to ArraySize do
  begin
    Survival[I, Random(ArraySize) + 1] := Random(SurvivalRandomLimit);
    Survival[I, Random(ArraySize) + 1] := Random(SurvivalRandomLimit)
  end;
  InitRuleGrids
end;

procedure TInteractingStatesDialog.GenerateNewRule;
begin
  if OptionsDialog.InteractingStatesBirth.Checked then
    begin
      BirthResetButtonClick(Self);
      BirthRandomButtonClick(Self);
    end;
    if OptionsDialog.InteractingStatesSurvival.Checked then
    begin
      SurvivalResetButtonClick(Self);
      SurvivalRandomButtonClick(Self);
    end;
end;

procedure TInteractingStatesDialog.OKButtonClick(Sender: TObject);
begin
  ModalResult := mrOK
end;

procedure TInteractingStatesDialog.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel
end;

procedure InitBirths;
var
  I, J: Integer;
begin
  for I := 1 to ArraySize do
    for J := 1 to ArraySize do
      Birth[I, J] := 0
end;

procedure InitSurvivals;
var
  I, J: Integer;
begin
  for I := 1 to ArraySize do
    for J := 1 to ArraySize do
      Survival[I, J] := 0
end;

procedure InitVars;
begin
  BirthTestOperator := OR_GE;
  SurvivalTestOperator := OR_GE;
  InitBirths;
  InitSurvivals
end;


function SaveRuleSet(S: String; K: Integer): Integer;
var
  T: String;
  I, J: Integer;
begin
  T := S[K];
  I := StrToInt(Copy(S, K + 1, 1));
  K := K + 3;
  J := 1;
  while (K <= Length(S)) and (S[K] <> ',') do
  begin
    if T = 'B' then
      Birth[I, J] := StrToInt(S[K])
    else
      Survival[I, J] := StrToInt(S[K]);
    Inc(J);
    Inc(K)
  end;
  Result := K + 1
end;

procedure InitRuleFromString(S: String);
var
  I: Integer;
begin
  if S = '' then
    InitVars
  else
  begin
    BirthTestOperator := TestOperator(StrToInt(S[3]));
    SurvivalTestOperator := TestOperator(StrToInt(S[7]));
    I := 9;
    while I < Length(S) do
      I := SaveRuleSet(S, I)
  end
end;

function GetRuleAsString: String;
var
  S: String;
  I, J: Integer;
begin
  S := 'TB' + IntToStr(Ord(BirthTestOperator)) + ',';
  S := S + 'TS' + IntToStr(Ord(SurvivalTestOperator)) + ',';
  for I := 1 to ArraySize do
  begin
    S := S + 'B' + IntToStr(I) + ';';
    for J := 1 to ArraySize do
      S := S + IntToStr(Birth[I, J]);
    S := S + ','
  end;
  for I := 1 to ArraySize do
  begin
    S := S + 'S' + IntToStr(I) + ';';
    for J := 1 to ArraySize do
      S := S + IntToStr(Survival[I, J]);
    if I <> ArraySize then
      S := S + ','
  end;
  Result := S
end;

end.
